// QuanLib:  eBook Library
// Copyright (C) 2017 by Christian Jaekl (cejaekl@yahoo.com)

'use strict';

// Global state information (yuck).  TODO:  refactor this to compartmentalize.
var g_state = {
    mousePos: {    // Last known position of the mouse cursor
        x: undefined,
        y: undefined
    }
};

// ==============
// Initialization

Browser.setOnMouseMove(onMouseMove);

BooksView.init(BooksModel);
PagingController.init(BooksModel);
SearchController.init(BooksModel);

if (Modernizr.promises) {
    console.log('quanweb:  browser feature check:  OK');
}
else {
    // If we cared about supporting older browsers (at this point, IE11 and Adroid 4.x's built-in browser,
    // neither of which is due to receive security patch support for much longer), then we would insert a 
    // shim here to implement promises.  But, in this case, we don't and won't.
    alert('Sorry, this page will not work in your browser.\nPlease use a recent version of Chrome, Edge, Firefox or Safari instead.');
}

// ================
// Global functions
// 
// TODO:  refactor this to compartmentalize more functionality.

function report(message) {
    Browser.getElementById('books').innerHTML = message;
}

function onMouseMove(event) {
    if (typeof event === 'undefined') {
        return;
    }
    
    var x = event.pageX;
    var y = event.pageY;
    
    if (  x === g_state.mousePos.x
       && y === g_state.mousePos.y)
    {
        // No change from previous known position. 
        // Nothing to see (or do) here, move along.
        return;
    }
    
    // Remember current mouse (x,y) position
    g_state.mousePos.x = x;
    g_state.mousePos.y = y;

    ToolTip.mouseMoved(x, y);
}

function onNext() {
    if (BooksModel.last < (BooksModel.count - 1)) {
        PagingController.adjustPos(BooksModel.first + BooksModel.pageSize);
    }
}

function onPrev() {
    if (BooksModel.first > 0) {
        PagingController.adjustPos(BooksModel.first - BooksModel.pageSize);
    }
}

function onSlide(value) {
    PagingController.adjustPos(value);
}

function onSearch() {
    SearchController.onSearch();
}

